﻿/**
* @class ColorMatch
* @author Viktor Goltvyanitsa
* @version 1.0.0.2
* @description You can convert the colors from one color model to another in this class.<br>
*Following are the supported conversions: RGB-CMY-CMYK-HSV-HLS.<p>
*<b>Availability:</b><br>
*        Flash Player 6 version 79 and above.<br>
*Action Script 2.0 </p>
*And get the chosen colors in several modes:
*byte - all values are displaying in the range of 0 - 255;
percent - all values are displaying as percentage 0 - 100%;
degree_byte - first value is displayed as an angle (0-360), others - in the range of 0 - 255;
degree_percent - first value is displayed as an angle (0-360), others - as percentage 0 - 100%;
Values in color models RGB and CMYK are displayed only as byte or percent.

You can correct the value of hue, saturation, lightness and brightness in chosen color, as well as shift it to the red, green or blue or add the shade of cyan, magenta, yellow on demand.
Also, you can get gradations of color branching (blender), web-safe colors array, nearest web-safe color and more.

You must use the constructor new ColorMatch() to create a ColorMatch object before calling its methods.
*/
import com.fwkit.color.HarmonySchemer;
import com.fwkit.utils.ArrayUtils;
import com.fwkit.color.ColorMatchCallBack;

class com.fwkit.color.ColorMatch {

	public static var version:String = "1.0.0.3";
	public static var Author:String = "Viktor Goltvyanitsa || http://www.fwkit.com";
	private static var hNum:Array = new Array("0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F");
	private static var colorTemp:Array = new Array("FF","CC","99","66","33","00");
	private static var NameColors:Array = new Array('AliceBlue','AntiqueWhite','Aqua','Aquamarine','Azure','Beige','Bisque','Black','BlanchedAlmond','Blue','BlueViolet','Brown','BurlyWood','CadetBlue','Chartreuse','Chocolate','Coral','CornflowerBlue','Cornsilk','Crimson','Cyan','DarkBlue','DarkCyan','DarkGoldenrod','DarkGray','DarkGreen','DarkGrey','DarkKhaki','DarkMagenta','DarkOliveGreen','DarkOrange','DarkOrchid','DarkRed','DarkSalmon','DarkSeaGreen','DarkSlateBlue','DarkSlateGray','DarkSlateGrey','DarkTurquoise','DarkViolet','DeepPink','DeepSkyBlue','DimGray','DimGrey','DodgerBlue','FireBrick','FloralWhite','ForestGreen','Fuchsia','Gainsboro','GhostWhite','Gold','Goldenrod','Gray','Grey','Green','GreenYellow','Honeydew','HotPink','IndianRed','Indigo','Ivory','Khaki','Lavender','LavenderBlush','LawnGreen','LemonChiffon','LightBlue','LightCoral','LightCyan','LightGoldenrodYellow','LightGray','LightGreen','LightGrey','LightPink','LightSalmon','LightSeaGreen','LightSkyBlue','LightSlateGray','LightSlateGrey','LightSteelBlue','LightYellow','Lime','LimeGreen','Linen','Magenta','Maroon','MediumAquamarine','MediumBlue','MediumOrchid','MediumPurple','MediumSeaGreen','MediumSlateBlue','MediumSpringGreen','MediumTurquoise','MediumVioletRed','MidnightBlue','MintCream','MistyRose','Moccasin','NavajoWhite','Navy','OldLace','Olive','OliveDrab','Orange','OrangeRed','Orchid','PaleGoldenrod','PaleGreen','PaleTurquoise','PaleVioletRed','PapayaWhip','PeachPuff','Peru','Pink','Plum','PowderBlue','Purple','Red','RosyBrown','RoyalBlue','SaddleBrown','Salmon','SandyBrown','SeaGreen','SeaShell','Sienna','Silver','SkyBlue','SlateBlue','SlateGray','SlateGrey','Snow','SpringGreen','SteelBlue','Tan','Teal','Thistle','Tomato','Turquoise','Violet','Wheat','White','WhiteSmoke','Yellow','YellowGreen');
	private static var NameRGB:Array = new Array('F0F8FF','FAEBD7','00FFFF','7FFFD4','F0FFFF','F5F5DC','FFE4C4','000000','FFEBCD','0000FF','8A2BE2','A52A2A','DEB887','5F9EA0','7FFF00','D2691E','FF7F50','6495ED','FFF8DC','DC143C','00FFFF','00008B','008B8B','B8860B','A9A9A9','006400','A9A9A9','BDB76B','8B008B','556B2F','FF8C00','9932CC','8B0000','E9967A','8FBC8F','483D8B','2F4F4F','2F4F4F','00CED1','9400D3','FF1493','00BFFF','696969','696969','1E90FF','B22222','FFFAF0','228B22','FF00FF','DCDCDC','F8F8FF','FFD700','DAA520','808080','808080','008000','ADFF2F','F0FFF0','FF69B4','CD5C5C','4B0082','FFFFF0','F0E68C','E6E6FA','FFF0F5','7CFC00','FFFACD','ADD8E6','F08080','E0FFFF','FAFAD2','D3D3D3','90EE90','D3D3D3','FFB6C1','FFA07A','20B2AA','87CEFA','778899','778899','B0C4DE','FFFFE0','00FF00','32CD32','FAF0E6','FF00FF','800000','66CDAA','0000CD','BA55D3','9370DB','3CB371','7B68EE','00FA9A','48D1CC','C71585','191970','F5FFFA','FFE4E1','FFE4B5','FFDEAD','000080','FDF5E6','808000','6B8E23','FFA500','FF4500','DA70D6','EEE8AA','98FB98','AFEEEE','DB7093','FFEFD5','FFDAB9','CD853F','FFC0CB','DDA0DD','B0E0E6','800080','FF0000','BC8F8F','4169E1','8B4513','FA8072','F4A460','2E8B57','FFF5EE','A0522D','C0C0C0','87CEEB','6A5ACD','708090','708090','FFFAFA','00FF7F','4682B4','D2B48C','008080','D8BFD8','FF6347','40E0D0','EE82EE','F5DEB3','FFFFFF','F5F5F5','FFFF00','9ACD32');
	private static var bluePalette:Array = new Array(80,135,121,17,92,130,35,96,101,21,88,44,41,77,129,67,117);
	private static var blueGreenPalette:Array = new Array(4,69,110,38,94,2,140,76,22,137,13);
	private static var greenPalette:Array = new Array(3,87,91,125,25,55,47,83,82,14,65,56,109,72,134,93,34,29,104,103);
	private static var yellowBrownPalette:Array = new Array(27,23,52,51,62,108,8,99,142,100,12,136,120,127,122,15,114,124,32,86,11,45);
	private static var orangeRedPalette:Array = new Array(59,68,123,33,75,16,139,30,105,106,19,119,40,48,58,74,115);
	private static var violetPurplePalette:Array = new Array(111,95,118,28,90,10,60,39,31,89,107,141,116,138);
	private static var pastelsPalette:Array = new Array(63,50,0,97,57,70,66,18,81,61,46,84,102,1,6,113,112,126,64,98,133);
	private static var primaryPalette:Array = new Array(119,145,82,20,9,85);
	private static var grayscalePalette:Array = new Array(143,144,49,71,128,24,53,42,7,36,131,78);
	private static var namePalettes:Array=["Blue","Blue Green","Green","Yellow Brown","Orange Red","Violet Purple","Pastels","Primary","Grayscale"];
	private static var namedIndexPalettes:Array=["bluePalette","blueGreenPalette","greenPalette","yellowBrownPalette","orangeRedPalette","violetPurplePalette","pastelsPalette","primaryPalette","grayscalePalette"];
	private static var schemNames:Array=["Complements","Split complements","Triads","Tetrads","Analogous","Monochromatic"];
	private var ArrayMatch:ArrayUtils;
	private static var colorF:Array = [];
	private static var r:Array = [];
	private static var g:Array = [];
	private static var b:Array = [];

	private var arrayMatch:ArrayUtils;
	private var callBack:ColorMatchCallBack;
	private var harmonySchemersHolder:Array=[];
	private var colorW:Array = [];
	private var colorS:Array = [];
	private var jj:Number = 0;

	private var name:String = null;
	private var pointColor:String = new String('');
	private var letter11:String = null;
	private var letter22:String = null;
	private var letter33:String = null;

	private var letterA = null;
	private var letterB = null;
	private var letterC = null;
	private var letterD = null;
	private var letterF = null;
	private var letterG = null;

	private var decR0:Number = null;
	private var decG0:Number = null;
	private var decB0:Number = null;

	private var Rdec:Number = null;
	private var Gdec:Number = null;
	private var Bdec:Number = null;

	private var colDec:Object = null;

	private var rrTemp2:Number = null;
	private var ggTemp2:Number = null;
	private var bbTemp2:Number = null;

	private var theDecTemp:Number = null;
	private var leftNum:Number = null;
	private var leftNumS:String = null;
	private var rightNum:Number = null;
	private var rightNumS:String = null;
	private var retNum:String = null;

	private var lightness:Number = null;
	private var rgb24:Number = null;
	private var o_r:Number = null;
	private var o_g:Number = null;
	private var o_b:Number = null;

	private var n_r:Number = null;
	private var n_g:Number = null;
	private var n_b:Number = null;

	private var r_f:String = null;
	private var g_f:String = null;
	private var b_f:String = null;
	private var x:String = null;

	private var i:Number = null;
	private var fi:Array = [];
	private var k:Number = null;
	private var num:Number = null;
	private var zz:Number = null;
	private var xx:Number = null;
	private var yy:Number = null;
	private var rrTemp3:Number = null;
	private var ggTemp3:Number = null;
	private var bbTemp3:Number = null;

	private var color_RGB:String = null;
	private var kra:Number = null;
	private var zel:Number = null;
	private var gol:Number = null;
	private var v1:Number = null;
	private var v2:Number = null;
	private var v3:Number = null;
	private var chrn:Number = null;
	private var color_CMYK:Object = null;

	private var out:String = null;

	private var endsR0:Number = null;
	private var endsG0:Number = null;
	private var endsB0:Number = null;
	private var endsR1:Number = null;
	private var endsG1:Number = null;
	private var endsB1:Number = null;
	private var number2:Number = null;
	private var step0:Number = null;
	private var step1:Number = null;
	private var step2:Number = null;
	private var j:Number = null;
	private var count:Number = null;
	private var r_a:Number = null;
	private var g_a:Number = null;
	private var b_a:Number = null;
	private var palette:Array = null;
	private var outBlender:Object = null;

	private var hh:Number = null;
	private var ss:Number = null;
	private var vv:Number = null;
	private var rem_1:Number = null;
	private var rem_2:Number = null;
	private var rem_3:Number = null;
	private var rem_4:Number = null;
	private var rem_5:Number = null;
	private var rem_6:Number = null;
	private var rem_7:Number = null;
	private var rem_8:Number = null;
	private var red_round:Number = null;
	private var gre_round:Number = null;
	private var blu_round:Number = null;
	private var out1:Number = null;

	private var rem:Number = null;
	private var rem1:Number = null;
	private var rem2:Number = null;
	private var r_hsv:Number = null;
	private var g_hsv:Number = null;
	private var b_hsv:Number = null;
	private var rem0:Number = null;
	private var rem01:Number = null;
	private var rem02:Number = null;
	private var rem3:Number = null;
	private var rem4:Number = null;
	private var rem5:Number = null;
	private var red_m:Number = null;
	private var gre_m:Number = null;
	private var blu_m:Number = null;
	private var hsv:Object = null;

	private var r_hls:Number = null;
	private var g_hls:Number = null;
	private var b_hls:Number = null;
	private var colorMax:Number = null;
	private var colorMin:Number = null;
	private var delta:Number = null;
	private var H_hls:Number = null;
	private var L_hls:Number = null;
	private var S_hls:Number = null;
	private var hls:Object = null;

	private var v1_2:Number = null;
	private var v2_2:Number = null;
	private var vH_2:Number = null;

	private var hh2:Number = null;
	private var ll2:Number = null;
	private var ss2:Number = null;
	private var R2:Number = null;
	private var G2:Number = null;
	private var B2:Number = null;
	private var var_1:Number = null;
	private var var_2:Number = null;
	private var out3:String = null;
	private var colorMax1:Number = null;
	private var colorMin1:Number = null;
	private var delta1:Number = null;
	private var Thr:Number = null;

	private var endsR3:Number = null;
	private var endsG3:Number = null;
	private var endsB3:Number = null;
	private var endsC3:Number = null;
	private var endsM3:Number = null;
	private var endsY3:Number = null;
	private var color_CMY:Object = null;

	private var C3:Number = null;
	private var M3:Number = null;
	private var Y3:Number = null;

	private var kra2:Number = null;
	private var zel2:Number = null;
	private var gol2:Number = null;
	private var out4:String = null;
	private var tempI:String = null;



	public function ColorMatch (_SETCOLOR:String){
			var thisArrayMatch:ArrayUtils=new ArrayUtils();
			arrayMatch=thisArrayMatch;
			setWSArray();
			setPointRGB(checkRGBString(_SETCOLOR));
			var thisArrayMatch:ArrayUtils=new ArrayUtils();
			ArrayMatch=thisArrayMatch;
		}

	public function setName(Name:String):Void{ name = Name }
	
	public function setCallBack(currentCallBack:ColorMatchCallBack):Void{
		if(currentCallBack!=undefined)callBack=currentCallBack;
	}
	
	public function deleteCallBack():Boolean{
		if(callBack==undefined){
			return false;
		}else{
			delete callBack;
			return true;
		}
	}

	public function getName(){ return name }

	public function setRGBstr(_SETCOLOR:String):Void {
		setPointRGB(checkRGBString(_SETCOLOR));	
		}

	public function getRGBstr() { return pointColor }

	public function getDec(Model:String){
		var decR0 = parseInt(pointColor.substr(0, 2), 16);
		var decG0 = parseInt(pointColor.substr(2, 2), 16);
		var decB0 = parseInt(pointColor.substr(4, 2), 16);
		switch(checkTwoModel(Model)){
			case "byte":
			var colDec = {Rdec:decR0, Gdec:decG0, Bdec:decB0};
			break;
			case "percent":
			var colDec = {Rdec:Math.round(decR0/2.55), Gdec:Math.round(decG0/2.55), Bdec:Math.round(decB0/2.55)};
		}
		return colDec;
	}

	public function getWebSafeArray() { return colorF }

	public function getWebSafeArrayDec(Component:String){
		switch(Component){
			case "red":
			return r;
			case "green":
			return g;
			case "blue":
			return b;
		}
		return r;
	}
	
	private function setPointRGB(RGBValue:String):Void{
		pointColor = RGBValue;
		if(callBack!=undefined)callBack.call();
	}

	public function setDec(rr2:Number, gg2:Number, bb2:Number, Model:String):Void{
		var rrTemp2  = rr2;
		var ggTemp2 = gg2;
		var bbTemp2 = bb2;
		switch(checkTwoModel(Model)){
			case "byte":
				rrTemp2 = Math.round(checkByte(rrTemp2));
				ggTemp2 = Math.round(checkByte(ggTemp2));
				bbTemp2 = Math.round(checkByte(bbTemp2));
			break;
			case "percent":
				rrTemp2 = Math.round(checkPercent(rrTemp2)*2.55);
				ggTemp2 = Math.round(checkPercent(ggTemp2)*2.55);
				bbTemp2 = Math.round(checkPercent(bbTemp2)*2.55);
		}
		setPointRGB(dec2hex(rrTemp2)+dec2hex(ggTemp2)+dec2hex(bbTemp2));
		delete retNum;
		}

	public function isWebSafe(){
		var letterA = colorTemp[0];
		var letterB = colorTemp[1];
		var letterC = colorTemp[2];
		var letterD = colorTemp[3];
		var letterF = colorTemp[4];
		var letterG = colorTemp[5];

		var letter11 = pointColor.substr(0, 2);
		var letter22 = pointColor.substr(2, 2);
		var letter33 = pointColor.substr(4, 2);

		if (letter11 == letterA || letter11 == letterB || letter11 == letterC || letter11 == letterD || letter11 == letterF || letter11 == letterG){
			if (letter22 == letterA || letter22 == letterB || letter22 == letterC || letter22 == letterD || letter22 == letterF || letter22 == letterG){
				if (letter33 == letterA || letter33 == letterB || letter33 == letterC || letter33 == letterD || letter33 == letterF || letter33 == letterG){
				return true;
			}else{
				return false;
				}
			}else{
				return false;
				}
			}else{
		return false;
		}
	}

	public function getColorWithLightness(Lt:Number){
		var lightness = checkPercent(Lt);
		var x = getBlender("FFFFFF", 100).nn;
		return x[lightness];
	}

	public function getColorWithDarkness(Dt:Number){
		var darkness = checkPercent(Dt);
		var x = getBlender("000000", 100).nn;
		return x[darkness];
	}

	public function getCMY(Model:String) {
		var endsR3 = parseInt(pointColor.substr(0, 2), 16);
		var endsG3 = parseInt(pointColor.substr(2, 2), 16);
		var endsB3 = parseInt(pointColor.substr(4, 2), 16);

		var C3 = 1 - endsR3/255;
		var M3 = 1 - endsG3/255;
		var Y3 = 1 - endsB3/255;

		switch(checkTwoModel(Model)){
		case "byte":
			var color_CMY = {C:Math.round(C3*255), M:Math.round(M3*255), Y:Math.round(Y3*255)}
		break;
		case "percent":
			var color_CMY = {C:Math.round(C3*100), M:Math.round(M3*100), Y:Math.round(Y3*100)};
		}
		delete endsC3;
		delete endsM3;
		delete endsY3;
		return color_CMY;
		}

	public function setCMY(colCMY:Object, Model:String):Void {
		switch(checkTwoModel(Model)){
		case "byte":
			var kra2 = 255-(checkByte(colCMY.C));
			var zel2 = 255-(checkByte(colCMY.M));
			var gol2 = 255-(checkByte(colCMY.Y));
			var out4 = dec2hex(Math.round(kra2))+dec2hex(Math.round(zel2))+dec2hex(Math.round(gol2));
		break;
		case "percent":
			var kra2 = 255-(checkPercent(colCMY.C)*2.55);
			var zel2 = 255-(checkPercent(colCMY.M)*2.55);
			var gol2 = 255-(checkPercent(colCMY.Y)*2.55);
			var out4 = dec2hex(Math.round(kra2))+dec2hex(Math.round(zel2))+dec2hex(Math.round(gol2));
		}
		setPointRGB(out4.toUpperCase());
	}

	public function getCMYK(Model:String) {
		var color_RGB = "0x"+pointColor;
		var kra = (color_RGB & 0xFF0000) >> 16;
		var zel = (color_RGB & 0x00FF00) >> 8;
		var gol = color_RGB & 0x0000FF;
		var v1 = 1-kra/255;
		var v2 = 1-zel/255;
		var v3 = 1-gol/255;
		var chrn = Math.min(v1, v2);
		chrn = Math.min(chrn, v3);
		switch(checkTwoModel(Model)){
			case "byte":
				var color_CMYK = {C:Math.round((100*(v1-chrn)/(1-chrn))*2.55), M:Math.round((100*(v2-chrn)/(1-chrn))*2.55), Y:Math.round((100*(v3-chrn)/(1-chrn))*2.55), K:Math.round((100*chrn)*2.55)};
			break;
		case "percent":
				var color_CMYK = {C:Math.round(100*(v1-chrn)/(1-chrn)), M:Math.round(100*(v2-chrn)/(1-chrn)), Y:Math.round(100*(v3-chrn)/(1-chrn)), K:Math.round(100*chrn)};
		}
		return color_CMYK;
	}

	public function setCMYK(colCMYK:Object, Model:String):Void {
		switch(checkTwoModel(Model)){
		case "byte":
			var chrn = checkByte(colCMYK.K)/255;
			var v1 = checkByte(colCMYK.C)/255;
			var v2 = checkByte(colCMYK.M)/255;
			var v3 = checkByte(colCMYK.Y)/255;
		break;
		case "percent":
			var chrn = checkPercent(colCMYK.K)/100;
			var v1 = checkPercent(colCMYK.C)/100;
			var v2 = checkPercent(colCMYK.M)/100;
			var v3 = checkPercent(colCMYK.Y)/100;
	}
		var kra = Math.round(255*(1-chrn+v1*(chrn-1)));
		var zel = Math.round(255*(1-chrn+v2*(chrn-1)));
		var gol = Math.round(255*(1-chrn+v3*(chrn-1)));
		var out = dec2hex(kra)+dec2hex(zel)+dec2hex(gol);
		setPointRGB(out.toUpperCase());
	}

	public function getBlender(colorTo:String, totalNumbers:Number) {
		if(isNaN(totalNumbers)) {totalNumbers = 1}
		if(totalNumbers<1) {totalNumbers=1}
		colorTo = checkRGBString(colorTo);

		var endsR0 = parseInt(pointColor.substr(0, 2), 16);
		var endsG0 = parseInt(pointColor.substr(2, 2), 16);
		var endsB0 = parseInt(pointColor.substr(4, 2), 16);

		var endsR1 = parseInt(colorTo.substr(0, 2), 16);
		var endsG1 = parseInt(colorTo.substr(2, 2), 16);
		var endsB1 = parseInt(colorTo.substr(4, 2), 16);

		var number2 = Math.round(totalNumbers)+1;

		var step0 = (endsR1-endsR0)/number2;
		var step1 = (endsG1-endsG0)/number2;
		var step2 = (endsB1-endsB0)/number2;

		var count = number2+1;

		var palette = new Array();
		palette[0] = pointColor;

		for (j=1; j<count; j++) {
			r_a = (endsR0+(step0*j));
			g_a = (endsG0+(step1*j));
			b_a = (endsB0+(step2*j));
			palette[j-1] = (dec2hex(Math.round(r_a))+dec2hex(Math.round(g_a))+dec2hex(Math.round(b_a)));
		}
		palette[count] = colorTo;

		var outBlender = {nn:palette};
		delete j;
		delete r_a;
		delete g_a;
		delete b_a;
		return outBlender;
	}

	public function setHSV(h:Number, s:Number, v:Number, Model:String):Void {
		var hh = h;
		var ss  = s;
		var vv = v;

		var rem_1 = 0;
		var rem_2 = 0;
		var rem_3 = 0;

		switch(checkFourModel(Model)){
		case "byte":
			if(hh==255) { hh = 0 };
			var rem_4 = Math.round((checkByte(hh)*1.411765))/60;
				vv = checkByte(vv)/255;
				ss = checkByte(ss)/255;
		break;
		case "percent":
		if(hh==100) { hh = 0 };
			var rem_4 = Math.round((checkPercent(hh)*3.6))/60;
			vv = checkPercent(vv)/100;
			ss = checkPercent(ss)/100;
			break;
			case "degree_percent":
			if(hh==360) { hh = 0 };
			var rem_4 = checkDegree(hh)/60;
			vv = checkPercent(vv)/100;
			ss = checkPercent(ss)/100;
			break;
			case "degree_byte":
			if(hh==360) { hh = 0 };
			var rem_4 = checkDegree(hh)/60;
			vv = checkByte(vv)/255;
			ss = checkByte(ss)/255;
		}

		var rem_5 = Math.floor(rem_4);
		var rem_7 = rem_4-rem_5;
		var rem_8 = (1-ss)*vv;

		rem_3 = (1-ss*rem_7)*vv;
		var rem_6 = (1-ss*(1-rem_7))*vv;

		switch (rem_5) {
		case 0 :
			rem_1 = vv;
			rem_2 = rem_6;
			rem_3 = rem_8;
		break;
		case 1 :
			rem_1 = rem_3;
			rem_2 = vv;
			rem_3 = rem_8;
		break;
		case 2 :
			rem_1 = rem_8;
			rem_2 = vv;
			rem_3 = rem_6;
		break;
		case 3 :
			rem_1 = rem_8;
			rem_2 = rem_3;
			rem_3 = vv;
		break;
		case 4 :
			rem_1 = rem_6;
			rem_2 = rem_8;
			rem_3 = vv;
		break;
		case 5 :
			rem_1 = vv;
			rem_2 = rem_8;
			rem_3 = rem_3;
		break;
		default :
			rem_1 = 0;
			rem_2 = 0;
			rem_3 = 0;
		}

		var red_round = Math.round(rem_1*255);
		var gre_round = Math.round(rem_2*255);
		var blu_round = Math.round(rem_3*255);
		var out1 = dec2hex(red_round)+dec2hex(gre_round)+dec2hex(blu_round);

		setPointRGB(out1.toUpperCase());
		}

		public function getHSV(Model:String) {
		var rem0 = 0;

		var r_hsv = parseInt(pointColor.substr(0, 2), 16);
		var g_hsv = parseInt(pointColor.substr(2, 2), 16);
		var b_hsv = parseInt(pointColor.substr(4, 2), 16);

		r_hsv /= 255;
		g_hsv /= 255;
		b_hsv /= 255;

		var rem3 = Math.max( Math.max(r_hsv, g_hsv), b_hsv);
			var rem4 = Math.min( Math.min(r_hsv, g_hsv), b_hsv);
			var rem01 = (rem3-rem4)/rem3;
			var rem02 = rem3;
			if (rem01 == 0) {
				rem0 = 0;
				} else {
				var rem5 = rem3-rem4;
					if (rem5 == 0) {
						rem0 = 0;
						rem01 = 0;
						} else {
								var red_m = (rem3-r_hsv)/rem5;
								var gre_m = (rem3-g_hsv)/rem5;
								var blu_m = (rem3-b_hsv)/rem5;
								if (r_hsv == rem3 && g_hsv == rem4) {
								rem0 = 5+blu_m;
							} else {
						if (r_hsv == rem3 && g_hsv != rem4) {
								rem0 = 1-gre_m;
							} else {
								if (g_hsv == rem3 && b_hsv == rem4) {
									rem0 = red_m+1;
									} else {
										if (g_hsv == rem3 && b_hsv != rem4) {
											rem0 = 3-blu_m;
											} else {
												if (r_hsv == rem4) {
													rem0 = 3+gre_m;
													} else {
														rem0 = 5-red_m;
														}
													}
												}
											}
										}
								rem0 *= 60;
							}
						}
		if (rem0>=360) { rem0 -= 360 }

		switch(checkFourModel(Model)){
			case "byte":
			var hsv = {H:Math.round((rem0/360)*255), S:Math.round(rem01*255), V:Math.round(rem02*255)}
			break;
			case "percent":
			var hsv = {H:Math.round(rem0/(360/100)), S:Math.round(rem01*100), V:Math.round(rem02*100)}
			break;
			case "degree_percent":
			var hsv = {H:Math.round(rem0), S:Math.round(rem01*100), V:Math.round(rem02*100)}
			break;
			case "degree_byte":
			var hsv = {H:Math.round(rem0), S:Math.round(rem01*255), V:Math.round(rem02*255)}
		}
		delete rem;
		delete rem1;
		delete rem2;

		return hsv;
	};


		public function getHLS(Model:String) {
			var r_hls = parseInt(pointColor.substr(0, 2), 16);
			var g_hls = parseInt(pointColor.substr(2, 2), 16);
			var b_hls = parseInt(pointColor.substr(4, 2), 16);

			var colorMax = (Math.max (Math.max ( r_hls, g_hls), b_hls))/255;
			var colorMin = (Math.min (Math.min ( r_hls, g_hls), b_hls))/255;
			var delta = colorMax-colorMin;
			var L_hls = (colorMax+colorMin)/2;

			if (colorMax==colorMin) {
				var S_hls = 0;
				} else if (L_hls<=0.5) {
					var S_hls = delta/L_hls/2;
					} else {
						var S_hls = delta/(2-L_hls*2);
						}
			if(colorMax==colorMin){
					H_hls = 0
				}else{
					var H_hls;
					var Thr = 0.0000001;
					var colorMax1 = Math.max (Math.max ( r_hls, g_hls), b_hls);
					var colorMin1 = Math.min (Math.min ( r_hls, g_hls), b_hls);
				var delta1  = colorMax1-colorMin1;
				if(delta1 < Thr){H_hls = 0}
				if(r_hls == colorMax1){
						H_hls = (g_hls - b_hls)/delta1;
					} else if(g_hls == colorMax1){
						H_hls = 2 + (b_hls - r_hls)/delta1;
						}else if(b_hls == colorMax1){
									H_hls = 4 + (r_hls - g_hls)/delta1;
							}
					H_hls = H_hls*60;
					if( H_hls < 0 ){ H_hls = H_hls +360}
				}

		switch(checkFourModel(Model)){
				case "byte":
				var hls = {H:Math.round(255*H_hls/360), L:Math.round(L_hls*255), S:Math.round(S_hls*255)};
				break;
				case "percent":
				var hls = {H:Math.round(H_hls/3.6), L:Math.round(L_hls*100), S:Math.round(S_hls*100)};
				break;
				case "degree_percent":
				var hls = {H:Math.round(H_hls), L:Math.round(L_hls*100), S:Math.round(S_hls*100)};
				break;
				case "degree_byte":
				var hls = {H:Math.round(H_hls), L:Math.round(L_hls*255), S:Math.round(S_hls*255)};
			}
		delete H_hls;
		return hls;
		}

		public function setHLS(hh:Number, ll:Number, ss:Number, Model:String):Void {
			var hh2 = hh;
			var ll2 = ll;
			var ss2 = ss;
			switch(checkFourModel(Model)){
				case "byte":
				hh2 = checkByte(hh2)/255;
				ll2 = checkByte(ll2)/255;
				ss2 = checkByte(ss2)/255;
				break;
			case "percent":
				hh2 = checkPercent(hh2)/100;
				ll2 = checkPercent(ll2)/100;
				ss2 = checkPercent(ss2)/100;
			break;
			case "degree_percent":
				hh2 = checkDegree(hh2)/360;
				ll2 = checkPercent(ll2)/100;
				ss2 = checkPercent(ss2)/100;
				break;
			case "degree_byte":
				hh2 = checkDegree(hh2)/360;
				ll2 = checkByte(ll2)/255;
				ss2 = checkByte(ss2)/255;
			}
		if (ss2 == 0) {
			var R2 = Math.round(ll2 * 255);
			var G2 = Math.round(ll2 * 255);
			var B2 = Math.round(ll2 * 255);
		}else{
			if (ll2 < 0.5) {
				var var_2 = ll2 *(1 + ss2);
			}
			else {var var_2 = (ll2 + ss2) - (ss2 * ll2)}
				var var_1 = 2*ll2 - var_2;
				R2 = Math.round(255 * Hue_2_RGB( var_1, var_2, hh2 + (1/3)));
				G2 = Math.round(255 * Hue_2_RGB( var_1, var_2, hh2    ));
				B2 = Math.round(255 * Hue_2_RGB( var_1, var_2, hh2 - (1/3)));
			}
			var out3 = dec2hex(R2)+dec2hex(G2)+dec2hex(B2);
			setPointRGB(out3.toUpperCase());
			}

		public function nearestWebSafe(){
			var decR0 = parseInt(pointColor.substr(0, 2), 16);
			var decG0 = parseInt(pointColor.substr(2, 2), 16);
			var decB0 = parseInt(pointColor.substr(4, 2), 16);
			for (var i=216; i; i--) {
				var rrTemp3  = r[i]-decR0;
				var ggTemp3 = g[i]-decG0;
				var bbTemp3 = b[i]-decB0;

				var zz  = 30*Math.pow(rrTemp3, 2);
				var yy = 59*Math.pow(ggTemp3, 2);
				var xx  = 11*Math.pow(bbTemp3, 2);

				fi[i] = zz+yy+xx;
				if (fi[i]<=k) {
				var k = fi[i];
				var num = i;
				}
			}
			return colorF[num];
	}
	
	public function getRandomRGBstr():String{
		var r_num:Number = Math.round(Math.random() * 255);
		var g_num:Number = Math.round(Math.random() * 255);
		var b_num = Math.round(Math.random() * 255);
		return dec2hex(r_num) + dec2hex(g_num) + dec2hex(b_num);
	}

	private function setWSArray(){
		if(colorF.length !=216){
			for (var ja=0; ja<6; ja++) {
				for (var jc=0; jc<6; jc++) {
					colorW[jc] = colorTemp[ja]+colorTemp[jc];
						for (var ji=0; ji<6; ji++) {
							colorS[ji] = colorW[jc]+colorTemp[ji];
							colorF[jj] = colorS[ji];
							jj = jj+1;
						}
					}
				}
		for (var ji=0; ji<216; ji++) {
			r[ji] = parseInt(colorF[ji].substr(0, 2), 16);
			g[ji] = parseInt(colorF[ji].substr(2, 2), 16);
			b[ji] = parseInt(colorF[ji].substr(4, 2), 16);
		}
			delete colorW;
			delete colorS;
		}
	}

	private function checkFourModel(Model:String){
		if(Model!="byte"&&Model!="percent"&&Model!="degree_percent"&&Model!="degree_byte"){
			return "byte";
		}else{
			return Model;
		}
	}

	private function checkTwoModel(Model:String){
		if(Model!="byte"&&Model!="percent"){
			return "byte";
		}else{
			return Model;
		}
	}

	public function checkByte(Value:Number){
		if(isNaN(Value)||Value<0) {
			return 0;
		}
		if(Value>255) {
			return 255;
		}
		return Value;
	}

	public function checkPercent(Value:Number){
		if(isNaN(Value)||Value<0) {
			return 0;
		}
		if(Value>100) {
			return 100;
		};
			return Value;
	}

	private function checkDegree(Value:Number){
		if(isNaN(Value)||Value<0) {
			return 0;
		}
		if(Value>360) {
			return 360;
		}
			return Value;
	}

	public function checkRGBString(Value:String){
		Value = Value.toUpperCase();
		if(Value.length>6){ Value = Value.substr(0, 6) }
		for (var i=0; i<Value.length; i++) {
			var tempI = Value.charAt(i);
			if(tempI!=="0"&&tempI!=="1"&&tempI!=="2"&&tempI!=="3"&&tempI!=="4"&&tempI!=="5"&&tempI!=="6"&&tempI!=="7"&&tempI!=="8"&&tempI!=="9"&&tempI!=="A"&&tempI!=="B"&&tempI!=="C"&&tempI!=="D"&&tempI!=="E"&&tempI!=="F"){
			return "000000";
			}
		}
		while(Value.length < 6) { Value = "0"+Value }
		return Value;
	}

	private function dec2hex(theDec:Number){
		var theDecTemp = theDec;
		if (theDecTemp>255) {theDecTemp = 255}
			var leftNum = Math.floor(theDecTemp/16);
			var leftNumS = hNum[leftNum];
			var rightNum = theDecTemp%16;
			var rightNumS = hNum[rightNum];
			var retNum = leftNumS+rightNumS;
		return retNum;
	}

	private function Hue_2_RGB(v1:Number, v2:Number, vH:Number) {
		var v1_2 = v1;
		var v2_2 = v2;
		var vH_2 = vH;
		if ( vH_2 < 0 ) {vH_2 += 1}
		if ( vH_2 > 1 ) {vH_2 -= 1}
		if (( 6 * vH_2 )< 1) {return ( v1_2 + (v2_2 - v1_2) * 6*vH_2 )}
		if (( 2 * vH_2 )< 1) {return ( v2_2 )}
		if (( 3 * vH_2 )< 2) {return ( v1_2 + (v2_2 - v1_2) * ((2/3) - vH_2)*6)}
		return (v1_2);
	}
	
	private function normalizeAngleOffset(angle:Number,offset:Number):Number{
		var normalizedAngle=angle+offset;
		if(normalizedAngle>360){
			while(normalizedAngle>360){
				normalizedAngle=normalizedAngle-360;
			}
			return normalizedAngle;
		}
		if(normalizedAngle<0){
			while(normalizedAngle<0){
				normalizedAngle=normalizedAngle+360;
			}
			return normalizedAngle;
		}
		return normalizedAngle;
	}
	
	private function normalizeDegreeOffset(degree:Number,offset:Number):Number{
		var normalizedDegree=degree+offset;
		if(normalizedDegree>100)return 100;
		if(normalizedDegree<0)return 0;
		return normalizedDegree;	
	}
	
	private function getRGBWithOffsetHue(thisOffset:Number):String{
		var complexVar:Object=getHLS("degree_percent");
		var thisH:Number=complexVar.H;
		var thisL:Number=complexVar.L;
		var thisS:Number=complexVar.S;
		var tempColorMatch:ColorMatch=new ColorMatch();
		var newH:Number=normalizeAngleOffset(thisH,thisOffset);
		tempColorMatch.setHLS(newH,thisL,thisS,"degree_percent");
		return tempColorMatch.getRGBstr();
	}
	
	private function getRGBWithOffsetLightness(thisOffset:Number):String{
		var complexVar:Object=getHLS("degree_percent");
		var thisH:Number=complexVar.H;
		var thisL:Number=complexVar.L;
		var thisS:Number=complexVar.S;
		var tempColorMatch:ColorMatch=new ColorMatch();
		var newL:Number=normalizeDegreeOffset(thisL,thisOffset);
		tempColorMatch.setHLS(thisH,newL,thisS,"degree_percent");
		return tempColorMatch.getRGBstr();
	}
	
	public function getHarmonyColors(Model:String):Array{
		var outArray:Array=[];
		switch(Model){
			case schemNames[0]:
			outArray.push(getRGBWithOffsetHue(180));
			return outArray;
			case schemNames[1]:
			outArray.push(getRGBWithOffsetHue(150));
			outArray.push(getRGBWithOffsetHue(210));
			return outArray;
			case schemNames[2]:
			outArray.push(getRGBWithOffsetHue(120));
			outArray.push(getRGBWithOffsetHue(240));
			return outArray;
			case schemNames[3]:
			outArray.push(getRGBWithOffsetHue(90));
			outArray.push(getRGBWithOffsetHue(180));
			outArray.push(getRGBWithOffsetHue(270));
			return outArray;
			case schemNames[4]:
			outArray.push(getRGBWithOffsetHue(30));
			outArray.push(getRGBWithOffsetHue(-30));
			return outArray;
			case schemNames[5]:
			outArray.push(getRGBWithOffsetLightness(10));
			outArray.push(getRGBWithOffsetLightness(20));
			return outArray;
			default:
			for(var i=0;i<6;i++){
				var tempArray:Array=new Array();
				tempArray=getHarmonyColors(schemNames[i]);
				var lengthClrs:Number=tempArray.length;
				for(var k=0;k<lengthClrs;k++){
					outArray.push(tempArray[k]);
				}
			}
			return outArray;				
		}
	}
	
	public function getAllHarmonyColorsRGBstr():Array{
			var harmonyArray:Array=new Array();
			harmonyArray=getHarmonyColors();
			harmonyArray.unshift(getRGBstr());	
			return harmonyArray;
	}
	
	public function createHarmonySchemer(holderClip:MovieClip,schemerWidth:Number,schemerHeight:Number):Boolean{
		var thisW:Number=schemerWidth||200;
		var thisH:Number=schemerHeight||120;
		var holderThisHarmony:Object=new Object();
			var currentSchemNames:Object=new Object();
		holderThisHarmony.Names=currentSchemNames;
		holderThisHarmony.Width=thisW;
		holderThisHarmony.Height=thisH;
		if(holderClip!=undefined&&typeof(holderClip)=="movieclip"){
			holderThisHarmony.Clip=holderClip;
			var hs:HarmonySchemer=new HarmonySchemer("harmonyBase",this);
			if(holderClip.harmonyBaseClip==undefined)holderClip.createEmptyMovieClip("harmonyBaseClip",holderClip.getNextHighestDepth());
			holderClip.harmonyBaseClip.harmonyClip=hs;
			holderClip.harmonyBaseClip.harmonyClip.setNameSchemes(currentSchemNames,schemNames);
			holderClip.harmonyBaseClip.harmonyClip.setClipLink(holderClip.harmonyBaseClip.harmonyClip);
			holderClip.harmonyBaseClip.harmonyClip.drawAll(holderClip.harmonyBaseClip,thisW,thisH,"DDDDDD",3,getAllHarmonyColorsRGBstr());
		}else{
			return false;
		}
		var numInHolder:Number=-1;
		var numItems:Number=harmonySchemersHolder.length;
		if(numItems>0){
			for(var i=0;i<numItems;i++){
				if(harmonySchemersHolder[i].Clip._name==holderThisHarmony.Clip._name){
					numInHolder=i;
				}
			}
		}
		if(numInHolder>0){
			harmonySchemersHolder[numInHolder]=holderThisHarmony;
		}else{
			harmonySchemersHolder.push(holderThisHarmony);
		}
	return true;
	}
	
	public function refreshHarmonySchemers():Void{
		var totalHarmonies=harmonySchemersHolder.length;
		if(totalHarmonies>0){
			var tempHolder:Array=[];
			tempHolder=arrayMatch.cloneArray(harmonySchemersHolder);
			for(var i=0;i<totalHarmonies;i++){
				createHarmonySchemer(tempHolder[i].Clip,tempHolder[i].Width,tempHolder[i].Height)
			}
		}		
	}
	
	public function deleteHarmonySchemer(holderClip:MovieClip):Boolean{
		if(holderClip.harmonyBaseClip!=undefined){
			holderClip.harmonyBaseClip.unloadMovie();
		}
		var totalHarmonies=harmonySchemersHolder.length;
		var indexItemForDelete:Number=-1;
		for(var i=0;i<totalHarmonies;i++){
			if(harmonySchemersHolder[i].Clip==holderClip){
				indexItemForDelete=i;
			}
		}
		if(indexItemForDelete>0){
			harmonySchemersHolder=arrayMatch.deleteItemByIndex(harmonySchemersHolder,indexItemForDelete);
			return true;
		}else{
			return false;
		}
	}
	
	public function setCoordinatesHarmonyShemer(holderClip:MovieClip,crdX:Number,crdY:Number):Boolean{
		if(holderClip.harmonyBaseClip!=undefined){
			var toX:Number=crdX||0;
			var toY:Number=crdY||0;
			holderClip.harmonyBaseClip._x=toX;
			holderClip.harmonyBaseClip._y=toY;
			return true;
		}else{
			return false;
		}
	}
	
	public function setVisibleHarmonyShemer(holderClip:MovieClip,mode:Boolean):Boolean{
		if(holderClip.harmonyBaseClip!=undefined){
			var thisMode:Boolean=mode||true;
			holderClip.harmonyBaseClip._visible=thisMode;
			return true;
		}else{
			return false;
		}
	}
	
	public function setSizeHarmonyShemer(holderClip:MovieClip,widthVal:Number,heightVal:Number):Boolean{
		if(holderClip.harmonyBaseClip!=undefined){
			var thisWidth:Number=widthVal||150;
			var thisHeight:Number=heightVal||200;
			holderClip.harmonyBaseClip.harmonyClip.resizeHarmony(thisWidth,thisHeight);
			return true;
		}else{
			return false;
		}
	}
	
	public function getNameColorPalettes():Array{
		return namePalettes;	
	}
	
	public function getColorPaletteByName(namePalette:String):Object{
		var outNames:Array=new Array();
		var outRGBVal:Array=new Array();
		var indexPal:Number=ArrayMatch.getIndex(namePalettes,namePalette);
	var _this=this;
		if(indexPal>=0){
		switch(indexPal){
			case 0:
			var outArrayIndex:Array=bluePalette;
			break;
			case 1:
			var outArrayIndex:Array=blueGreenPalette;
			break;
			case 2:
			var outArrayIndex:Array=greenPalette;
			break;
			case 3:
			var outArrayIndex:Array=yellowBrownPalette;
			break;
			case 4:
			var outArrayIndex:Array=orangeRedPalette;
			break;
			case 5:
			var outArrayIndex:Array=violetPurplePalette;
			break;
			case 6:
			var outArrayIndex:Array=pastelsPalette;
			break;
			case 7:
			var outArrayIndex:Array=primaryPalette;
			break;
			case 8:
			var outArrayIndex:Array=grayscalePalette;
			break;
		}
			var totalEl:Number=outArrayIndex.length;
			for(var i:Number=0;i<totalEl;i++){
				var thisIndex:Number=outArrayIndex[i];
				outNames.push(NameColors[thisIndex]);
				outRGBVal.push(NameRGB[thisIndex]);
			}
		}
		var outObj={colorNames:outNames,colorVals:outRGBVal};
		return outObj;
	}
	
}